@@ -1,22 +1,22 @@
 /* GNU m4 -- A simple macro processor
 
-   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2006, 2007
-   Free Software Foundation, Inc.
+   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2006, 2007, 2008,
+   2009 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This file is part of GNU M4.
+
+   GNU M4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   GNU M4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301  USA
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* printf like formatting for m4.  */
@@ -41,7 +41,7 @@
 
 #define ARG_DOUBLE(argc, argv) \
 	((argc == 0) ? 0 : \
-	 (--argc, argv++, atof (TOKEN_DATA_TEXT (argv[-1]))))
+	 (--argc, argv++, strtod (TOKEN_DATA_TEXT (argv[-1]), NULL)))
 
 
 /*------------------------------------------------------------------.
@@ -53,7 +53,7 @@
 `------------------------------------------------------------------*/
 
 void
-format (struct obstack *obs, int argc, token_data **argv)
+expand_format (struct obstack *obs, int argc, token_data **argv)
 {
   const char *f;			/* format control string */
   const char *fmt;			/* position within f */
@@ -89,14 +89,16 @@ format (struct obstack *obs, int argc, token_data **argv)
 
   f = fmt = ARG_STR (argc, argv);
   memset (ok, 0, sizeof ok);
-  for (;;)
+  while (1)
     {
-      while ((c = *fmt++) != '%')
+      const char *percent = strchr (fmt, '%');
+      if (!percent)
 	{
-	  if (c == '\0')
-	    return;
-	  obstack_1grow (obs, c);
+	  obstack_grow (obs, fmt, strlen (fmt));
+	  return;
 	}
+      obstack_grow (obs, fmt, percent - fmt);
+      fmt = percent + 1;
 
       if (*fmt == '%')
 	{
